@@ -13,6 +13,7 @@ import requests
 import psutil
 
 from turcar.check_for_updates import CheackUpdates
+from turcar.login import UserLogin
 
 from turcar.common import is_private_python, is_virtual_executable
 
@@ -328,7 +329,7 @@ def launch():
                         # else:  # 默认为Ubuntu或其他Linux发行版
                         cheack = CheackUpdates()
                         # 添加判断是为了区分，否则会点什么都退出
-                        if cheack.cheack_version(version) == True:
+                        if cheack.cheack_version(version):
                             return
         else:
             # 后端返回不等于两百的逻辑
