@@ -640,7 +640,7 @@ class BaseFileBrowser(ttk.Frame):
         # get_workbench().show_view("PdfView", set_focus=False)
         # get_workbench().event_generate("PdfView", content=path)
     def video_play(self, path):
-        get_workbench().show_view("VideoPlay")
+        get_workbench().show_view("VideoPlay", set_focus=False)
         get_workbench().event_generate("VideoPlay", path=path)
     def post_button_menu(self):
         self.refresh_menu(context="button")
