@@ -1,24 +1,25 @@
 # -*- coding: utf-8 -*-
 import base64
 import datetime
+import hashlib
 import json
 import os.path
 import re
+import os
 import sys
 import tkinter as tk
 import platform
-import traceback
 import urllib
 from logging import exception, getLogger
 from tkinter import messagebox, simpledialog, ttk
 from typing import Optional, Union
-import cv2
+# import cv2
 import requests
 from _tkinter import TclError
-
 from turcar import get_runner, get_workbench, ui_utils
 from turcar.base_file_browser import ask_backend_path, choose_node_for_file_operations
 from turcar.codeview import BinaryFileException, CodeView, CodeViewText
+from tkinter import ttk
 from turcar.common import (
     REMOTE_PATH_MARKER,
     InlineCommand,
@@ -39,10 +40,8 @@ from turcar.ui_utils import (
     asksaveasfilename,
     get_beam_cursor,
     select_sequence,
-    windows_known_extensions_are_hidden,
 )
-from turcar.video import VideoDialog
-
+# from cefpython3 import cefpython as cef
 PYTHON_FILES_STR = tr("Python files")
 _dialog_filetypes = [(PYTHON_FILES_STR, ".py .pyw .pyi"), (tr("all files"), ".*")]
 
@@ -776,6 +775,27 @@ class EditorNotebook(CustomNotebook):
             group=60,
         )
 
+        get_workbench().add_command(
+            "watch_video",
+            "course",
+            tr("课程更新"),
+            self._cmd_course_update,
+            caption=tr("Video"),
+            group=10,
+            image="download1",
+            include_in_toolbar=True,
+        )
+        # get_workbench().add_command(
+        #     "programming_exercises",
+        #     "programming",
+        #     tr("课后练习"),
+        #     self._cmd_programming_exercises,
+        #     caption=tr("Programming"),
+        #     group=10,
+        #     image="download1",
+        #     include_in_toolbar=True,
+        # )
+
         # get_workbench().add_command(
         #     "watch_video",
         #     "course",
@@ -812,6 +832,264 @@ class EditorNotebook(CustomNotebook):
 
         get_workbench().createcommand("::tk::mac::OpenDocument", self._mac_open_document)
 
+    def count_files(self, data):
+        count = 0
+        for item in data:
+            if 'file_url' in item:
+                count += 1
+            else:
+                count += self.count_files(item.get('childer', []))
+        return count
+
+    def check_path(self, path):
+        if os.path.isdir(path):
+            return False
+        elif os.path.isfile(path):
+            return True
+        else:
+            return False
+
+    def calculate_md5(self, file_path):
+        with open(file_path, 'rb') as file:
+            md5 = hashlib.md5()
+            while True:
+                chunk = file.read(8192)
+                if not chunk:
+                    break
+                md5.update(chunk)
+        return md5.hexdigest()
+
+    def update_files_count(self, remote_data, parent_path):
+        for remote_item in remote_data:
+            remote_name = remote_item['name']
+            remote_childer = remote_item.get('childer', [])
+            file_name = os.path.join(parent_path, remote_name)
+            if not os.path.exists(file_name):
+                if 'file_url' in remote_item:
+                    self.cy_count += 1
+            else:
+                check_file = self.check_path(file_name)
+                # if check_file:
+                #     print('计算的md5', self.calculate_md5(file_name))
+            if len(remote_childer):
+                self.update_files_count(remote_childer, os.path.join(parent_path, remote_name))
+
+    def check_file(self, path):
+        if os.path.isdir(path):
+            return False
+        else:
+            return True
+
+    def remove_files(self, remote_data, parent_path):
+        contents = os.listdir(parent_path)
+        for item in contents:
+            is_file = self.check_file(os.path.join(parent_path, item))
+            if is_file:
+                flag = False
+                for i in remote_data:
+                    if i['name'] == item:
+                        flag = True
+                if not flag:
+                    os.remove(os.path.join(parent_path, item))
+            else:
+                flag = False
+                for i in remote_data:
+                    if i['name'] == item:
+                        flag = True
+                        res_path = i.get('childer', [])
+                        self.remove_files(res_path, os.path.join(parent_path, item))
+                if not flag:
+                    self.delete_directory_recursive(os.path.join(parent_path, item))
+
+    def delete_directory_recursive(self,  directory_path):
+        if os.path.isdir(directory_path):
+            for item in os.listdir(directory_path):
+                item_path = os.path.join(directory_path, item)
+                if os.path.isfile(item_path):
+                    os.remove(item_path)
+                elif os.path.isdir(item_path):
+                    self.delete_directory_recursive(item_path)
+            os.rmdir(directory_path)
+            print(f"文件夹 {directory_path} 已删除")
+        else:
+            print(f"文件夹 {directory_path} 不存在")
+
+
+    def update_files(self, remote_data, parent_path):
+        for remote_item in remote_data:
+            remote_name = remote_item['name']
+            remote_childer = remote_item.get('childer', [])
+            file_name = os.path.join(parent_path, remote_name)
+            if not os.path.exists(file_name):
+                if not 'file_url' in remote_item:
+                    os.makedirs(file_name, exist_ok=True)
+                else:
+                    self.download_file(remote_item, file_name, str(str(self.cy_current) + '/' + str(self.cy_count)) )
+                    self.cy_current += 1
+            else:
+                check_file = self.check_path(file_name)
+                # if check_file:
+                #     print('计算的md5', self.calculate_md5(file_name))
+
+            if len(remote_childer):
+                self.update_files(remote_childer, os.path.join(parent_path, remote_name))
+
+    def check_internet_connection(self):
+        try:
+            response = requests.post(self.pull_url)
+            if response.status_code == 200:
+                return True
+            else:
+                return False
+        except requests.RequestException as e:
+            return False
+
+    def _cmd_course_update(self):
+        self.pull_url = 'http://pyai.yinhangjihua.cn/api/exam/api/sys/fileUpload/first-update'
+        if not self.check_internet_connection():
+            return messagebox.showerror('无网络', '网络连接失败')
+
+        current_os = platform.system()
+        if current_os == "Windows":
+            file_path = 'D:\\course'
+        else:
+            course_name = 'course'
+            file_path = os.popen("echo $HOME/"+course_name).read().strip()
+        if not os.path.exists(file_path):
+            os.makedirs(file_path)
+        files = os.listdir(file_path)
+        data = requests.post(self.pull_url)
+        resp = data.json()['data']['courseDirectoryDTOS']
+        self.remote_list = []
+        for item in resp:
+            chapter_list = item['chapterDirectoryDTOS']
+            chapter_list_res = []
+            for i in chapter_list:
+                content_list = i['contentDTOS']
+                content_list_res = []
+                for ci in content_list:
+                    content_list_res.append({
+                        "name": ci['name'],
+                        "file_url": ci['path']
+                    })
+                chapter_list_res.append({
+                    "name": i['title'],
+                    'childer': content_list_res
+                })
+            self.remote_list.append({
+                "name": item['title'],
+                "childer": chapter_list_res
+            })
+        count = self.count_files(self.remote_list)
+        if not files:
+            # 第一次，全量更新
+            self.current = 1
+            message_res = messagebox.askyesno('确定更新', '你确定要更新吗？当前为全量更新，需更新'+str(count) + '个文件')
+            if message_res:
+                self.create_folder(self.remote_list, file_path, count, self.current)
+        else:
+            # 差异更新
+            self.cy_count = 0
+            self.cy_current = 1
+            self.remove_files(self.remote_list, file_path)
+            self.update_files_count(self.remote_list, file_path)
+            if self.cy_count == 0:
+                return messagebox.showinfo('无', '暂无更新')
+            else:
+                message_res = messagebox.askyesno('确定更新',
+                                                  '你确定要更新吗？当前为差异更新，需更新' + str(self.cy_count) + '个文件')
+                if message_res:
+                    self.update_files(self.remote_list, file_path)
+
+    # def _cmd_programming_exercises(self):
+    #     sys.excepthook = cef.ExceptHook  # To shutdown all CEF processes on error
+    #     cef.Initialize()
+    #     cef.CreateBrowserSync(url="http://system.turcar.net.cn",
+    #                           window_title="Hello World!")
+    #     cef.MessageLoop()
+    #     cef.Shutdown()
+        # from programming_exercises1 import MainFrame
+        # MainFrame(self)
+
+    def download_file(self, item, folder_path, str):
+        response = requests.get(item['file_url'], stream=True)
+        with open(folder_path, 'wb') as f:
+            total_length = response.headers.get('content-length')
+            if total_length is None:  # 没有内容长度头
+                f.write(response.content)
+            else:
+                self.create_download_progress_view(total_length, f, item, response,
+                                                   str)
+
+    def create_folder(self, data, parent_path, count, current):
+        for item in data:
+            folder_path = os.path.join(parent_path, item['name'])
+            if 'file_url' in item:
+                # 如果有 file_url 就是文件，直接下载保存到 folder_path
+                self.download_file(item, folder_path, str(str(self.current) + '/' + str(count)))
+                self.current += 1
+            else:
+                # 否则就是文件夹，先创建文件夹，再递归调用 create_folder
+                os.makedirs(folder_path, exist_ok=True)
+                self.create_folder(item['childer'], folder_path, count, self.current)
+
+    def show(self, progress, text):
+        if progress == '100.00':
+            try:
+                self.root.destroy()
+            except:
+                pass
+        else:
+            self.progressbarOne['value'] = progress
+            self.label.config(text=text)
+            self.root.update()
+
+    def null_def(self):
+        messagebox.showerror('错误', '禁止关闭')
+    def create_download_progress_view(self, total_length, f, item, response, count):
+        self.root = tk.Tk()
+        screen_width = self.root.winfo_screenwidth()
+        screen_height = self.root.winfo_screenheight()
+        window_width = 800
+        window_height = 200
+        window_x = int(screen_width / 2 - window_width / 2)
+        window_y = int(screen_height / 2 - window_height / 2)
+        self.root.geometry(f"{window_width}x{window_height}+{window_x}+{window_y}")
+        self.root.title('资料更新')
+        self.root.protocol("WM_DELETE_WINDOW", self.null_def)
+        self.div = tk.Frame(self.root)
+        self.div.place(relx=0.5,rely=0.5,anchor='center')
+
+        self.progressbarOne = ttk.Progressbar(self.div)
+
+
+        self.progressbarOne.pack()
+        # 进度值最大值
+        self.progressbarOne['maximum'] = 100
+        # 进度值初始值
+        self.progressbarOne['value'] = 0
+        self.progressbarOne['length'] = 480
+
+        self.label = tk.Label(self.div)
+        self.label.pack(pady=20)
+
+        self.label1 = tk.Label(self.div, text=count)
+        self.label1.pack(pady=10)
+
+        downloaded = 0
+        total_length = int(total_length)
+        for data in response.iter_content(chunk_size=4096):
+            downloaded += len(data)
+            f.write(data)
+            status = response.status_code
+            if status != 200:
+                print(f"下载失败: {status}")
+                break
+            if total_length != 0:
+                pro = "{:.2f}".format(downloaded / total_length * 100)
+            else:
+                pro = "0.00"
+            self.show(pro, '当前下载文件为--' + item['name'] + '--,' + "进度为:--" + str(pro) + '%--')
     def load_startup_files(self):
         """If no filename was sent from command line
         then load previous files (if setting allows)"""
@@ -861,6 +1139,7 @@ class EditorNotebook(CustomNotebook):
         recents.insert(0, filename)
         relevant_recents = [name for name in recents if os.path.exists(name)][:15]
         get_workbench().set_option("file.recent_files", relevant_recents)
+        get_workbench().set_option('general.ui_mode', 'regular')
         self._update_recent_menu()
 
     def _update_recent_menu(self):
@@ -955,7 +1234,7 @@ class EditorNotebook(CustomNotebook):
         else:
             page = self.get_page_by_tab(index_or_tab)
 
-        assert isinstance(page.content, Editor)
+        # assert isinstance(page.content, Editor)
         self.close_editor(page.content)
 
     def close_editor(self, editor, force=False):
@@ -1126,92 +1405,91 @@ class EditorNotebook(CustomNotebook):
             print('取消关机了')
     # 人脸识别
     def _cmd_face_recognition(self):
-        # 获取当前操作系统
-        current_os = platform.system()
-
-        if current_os == "Windows":
-            save_path = "D:/copy.png"
-            cap = cv2.VideoCapture(0, cv2.CAP_DSHOW)  # 打开笔记本内置摄像头
-        if current_os == "Darwin":  # 默认为Ubuntu或其他Linux发行版
-            save_path = "../res/copy.png"
-            cap = cv2.VideoCapture(0)
-        else:
-            save_path = "res/copy.png"
-            cap = cv2.VideoCapture(0)
-            # todo Ubuntu摄像头获取失败
-            # pass
-
-        while cap.isOpened():  # 笔记本内置摄像头被打开后
-            ret, frame = cap.read()  # 从摄像头中实时读取视频
-            cv2.imshow("Recognition", frame)  # 在窗口中显示视频
-            k = cv2.waitKey(1)  # 图像的刷新时间为1毫秒
-            if k == ord(' '):  # 按下空格键
-                cv2.imshow('img', frame)  # 显示按下空格键时摄像头视频中的图像
-                cv2.imwrite(save_path, frame)  # 保存按下空格键时摄像头视频中的图像
-                cap.release()  # 关闭笔记本内置摄像头
-                cv2.destroyWindow("Recognition")  # 销毁名为Video的窗口
-                break
-        cv2.destroyAllWindows()  # 销毁显示图像的窗口
-        url = "https://aip.baidubce.com/rest/2.0/face/v3/detect?access_token=" + get_access_token()
-
-        # image 可以通过 get_file_content_as_base64("C:\fakepath\多人二维码.jpg",False) 方法获取
-        # path = r"D:/copy.png"
-        image = get_file_content_as_base64(save_path, False)
-        payload = json.dumps({
-            "image": image,
-            "image_type": "BASE64",
-            "face_field": "age,expression,face_shape,gender,glasses,emotion,mask",
-            "max_face_num": 100
-        })
-        headers = {
-            'Content-Type': 'application/json'
-        }
-        response = requests.request("POST", url, headers=headers, data=payload)
-        # 解析response为JSON格式
-        response_data = json.loads(response.text)
-        error_code = response_data["error_code"]
-        timestamp = response_data["timestamp"]
-        dt = datetime.datetime.fromtimestamp(timestamp)
-        # 将datetime对象转换为指定格式的字符串
-        formatted_datetime = dt.strftime("%Y-%m-%d %H:%M:%S")
-        root = tk.Tk()
-        root.withdraw()  # 隐藏主窗口
-        if error_code == 0:
-            # 获取face_num等字段
-            face_num = response_data['result']['face_num']
-            face_list_text = f"人脸信息列表：\n"
-            if face_num > 0:
-                face_list = response_data['result']['face_list']
-                print(face_list)
-                # 遍历列表中的字典
-                for item in face_list:
-                    face_probability = item['face_probability']
-                    age = item["age"]
-                    expression = self.determine_expression(item['expression'])
-                    face_shape = self.determine_face_shape(item['face_shape'])
-                    gender = self.determine_gender(item['gender']['type'])
-                    glasses = self.determine_glasses(item['glasses'])
-                    emotion = self.determine_emotion(item['emotion'])
-                    mask = self.determine_mask(item['mask'])
-                    item_text = f"[\n" \
-                                f" 人脸置信度（0—1）：{face_probability}\n" \
-                                f" 年龄：{age}\n" \
-                                f" 表情：{expression}\n" \
-                                f" 脸型：{face_shape}\n" \
-                                f" 性别：{gender}\n" \
-                                f" 是否戴眼镜：{glasses}\n" \
-                                f" 情绪：{emotion}\n" \
-                                f" 口罩：{mask}\n" \
-                                f"]\n"
-                    face_list_text = face_list_text + item_text
-
-            messagebox.askokcancel("识别结果", f"识别成功,时间：{formatted_datetime}"
-                                               f"\n检测到的图片中的人脸数量：{face_num}"
-                                               f"\n{face_list_text}")
-
-        else:
-            messagebox.askokcancel("识别结果", "请求失败")
-        return response
+        pass
+        # # 获取当前操作系统
+        # current_os = platform.system()
+        #
+        # if current_os == "Windows":
+        #     save_path = "D:/copy.png"
+        #     cap = cv2.VideoCapture(0, cv2.CAP_DSHOW)  # 打开笔记本内置摄像头
+        # if current_os == "Darwin":  # 默认为Ubuntu或其他Linux发行版
+        #     save_path = "turcar/copy.png"
+        #     cap = cv2.VideoCapture(0)
+        # else:
+        #     # todo Ubuntu摄像头获取失败
+        # pass
+        #
+        # while cap.isOpened():  # 笔记本内置摄像头被打开后
+        #     ret, frame = cap.read()  # 从摄像头中实时读取视频
+        #     cv2.imshow("Recognition", frame)  # 在窗口中显示视频
+        #     k = cv2.waitKey(1)  # 图像的刷新时间为1毫秒
+        #     if k == ord(' '):  # 按下空格键
+        #         cv2.imshow('img', frame)  # 显示按下空格键时摄像头视频中的图像
+        #         cv2.imwrite(save_path, frame)  # 保存按下空格键时摄像头视频中的图像
+        #         cap.release()  # 关闭笔记本内置摄像头
+        #         cv2.destroyWindow("Recognition")  # 销毁名为Video的窗口
+        #         break
+        # cv2.destroyAllWindows()  # 销毁显示图像的窗口
+        # url = "https://aip.baidubce.com/rest/2.0/face/v3/detect?access_token=" + get_access_token()
+        #
+        # # image 可以通过 get_file_content_as_base64("C:\fakepath\多人二维码.jpg",False) 方法获取
+        # # path = r"D:/copy.png"
+        # image = get_file_content_as_base64(save_path, False)
+        # payload = json.dumps({
+        #     "image": image,
+        #     "image_type": "BASE64",
+        #     "face_field": "age,expression,face_shape,gender,glasses,emotion,mask",
+        #     "max_face_num": 100
+        # })
+        # headers = {
+        #     'Content-Type': 'application/json'
+        # }
+        # response = requests.request("POST", url, headers=headers, data=payload)
+        # # 解析response为JSON格式
+        # response_data = json.loads(response.text)
+        # error_code = response_data["error_code"]
+        # timestamp = response_data["timestamp"]
+        # dt = datetime.datetime.fromtimestamp(timestamp)
+        # # 将datetime对象转换为指定格式的字符串
+        # formatted_datetime = dt.strftime("%Y-%m-%d %H:%M:%S")
+        # root = tk.Tk()
+        # root.withdraw()  # 隐藏主窗口
+        # if error_code == 0:
+        #     # 获取face_num等字段
+        #     face_num = response_data['result']['face_num']
+        #     face_list_text = f"人脸信息列表：\n"
+        #     if face_num > 0:
+        #         face_list = response_data['result']['face_list']
+        #         print(face_list)
+        #         # 遍历列表中的字典
+        #         for item in face_list:
+        #             face_probability = item['face_probability']
+        #             age = item["age"]
+        #             expression = self.determine_expression(item['expression'])
+        #             face_shape = self.determine_face_shape(item['face_shape'])
+        #             gender = self.determine_gender(item['gender']['type'])
+        #             glasses = self.determine_glasses(item['glasses'])
+        #             emotion = self.determine_emotion(item['emotion'])
+        #             mask = self.determine_mask(item['mask'])
+        #             item_text = f"[\n" \
+        #                         f" 人脸置信度（0—1）：{face_probability}\n" \
+        #                         f" 年龄：{age}\n" \
+        #                         f" 表情：{expression}\n" \
+        #                         f" 脸型：{face_shape}\n" \
+        #                         f" 性别：{gender}\n" \
+        #                         f" 是否戴眼镜：{glasses}\n" \
+        #                         f" 情绪：{emotion}\n" \
+        #                         f" 口罩：{mask}\n" \
+        #                         f"]\n"
+        #             face_list_text = face_list_text + item_text
+        #
+        #     messagebox.askokcancel("识别结果", f"识别成功,时间：{formatted_datetime}"
+        #                                        f"\n检测到的图片中的人脸数量：{face_num}"
+        #                                        f"\n{face_list_text}")
+        #
+        # else:
+        #     messagebox.askokcancel("识别结果", "请求失败")
+        # return response
 
     def determine_gender(self, gender):
         return '男' if gender == 'male' else '女'
